library ac_dart.descriptor_model.hipchat;

import 'package:atlassian_connect_host/src/json_utils.dart';

class HipchatConnectAddon {
  final String key;
  final String name;
  final String description;
  final Capabilities capabilities;

  HipchatConnectAddon(
      {this.key, this.name, this.description, this.capabilities});

  factory HipchatConnectAddon.fromJson(Map json) {
    final p = parseJson(json);
    return new HipchatConnectAddon(
        key: p.single('key'),
        name: p.single('name'),
        description: p.single('description'),
        capabilities: p.single(
            'capabilities', (v) => new Capabilities.fromJson(v)));
  }

  Map toJson() => (buildJson
    ..add('key', key)
    ..add('name', name)
    ..add('description', description)
    ..add('capabilities', capabilities)).json;
}

class Capabilities {
  final Installable installable;
  final Configurable configurable;
  final HipchatApiConsumer hipchatApiConsumer;

  Capabilities({this.installable, this.configurable, this.hipchatApiConsumer});

  factory Capabilities.fromJson(Map json) {
    final p = parseJson(json);
    return new Capabilities(
        installable: p.single(
            'installable', (v) => new Installable.fromJson(v)),
        configurable: p.single(
            'configurable', (v) => new Configurable.fromJson(v)),
        hipchatApiConsumer: p.single(
            'hipchatApiConsumer', (v) => new HipchatApiConsumer.fromJson(v)));
  }

  Map toJson() => (buildJson
    ..add('installable', installable)
    ..add('configurable', configurable)
    ..add('hipchatApiConsumer', hipchatApiConsumer)).json;
}

class Installable {
  final bool allowGlobal;
  final bool allowRoom;
  final Uri callbackUri;

  Installable(this.allowGlobal, this.allowRoom, this.callbackUri);

  factory Installable.fromJson(Map json) {
    final p = parseJson(json);
    return new Installable(p.single('allowGlobal'), p.single('allowRoom'),
        p.single('callbackUri', (v) => Uri.parse(v)));
  }

  Map toJson() => (buildJson
    ..add('allowGlobal', allowGlobal)
    ..add('allowRoom', allowRoom)
    ..add('callbackUri', callbackUri)).json;

  Installable copy({Uri callbackUri}) => new Installable(allowGlobal, allowRoom,
      callbackUri != null ? callbackUri : this.callbackUri);
}

class Configurable {
  final bool allowAccessToRoomAdmins;
  final Uri url;

  Configurable(this.allowAccessToRoomAdmins, this.url);

  factory Configurable.fromJson(Map json) {
    final p = parseJson(json);
    return new Configurable(p.single('allowAccessToRoomAdmins'),
        p.single('url', (v) => Uri.parse(v)));
  }

  Map toJson() => (buildJson
    ..add('allowAccessToRoomAdmins', allowAccessToRoomAdmins)
    ..add('callbackUri', url)).json;

  Configurable copy({Uri url}) =>
      new Configurable(allowAccessToRoomAdmins, url != null ? url : this.url);
}

class HipchatApiConsumer {
  final Iterable<String> scopes;

  HipchatApiConsumer(this.scopes);

  factory HipchatApiConsumer.fromJson(Map json) {
    final p = parseJson(json);
    return new HipchatApiConsumer(p.list('scopes'));
  }

  Map toJson() => (buildJson..add('scopes', scopes)).json;
}
